from meta import util
from combat import spell_classes
import time
import random


class Combat(object):
    def __init__(self, PC, mob):
        # Meta
        self.player = PC
        self.mob = mob
        self.spell_invoker = spell_classes.SpellInvoker(self.player, self.mob)
        self.turn_counter = 0

        # Turn
        self.player_speed = 0
        self.player_speed_mod = 0
        self.player_crit_chance = 0
        self.player_def_mod = 0
        self.player_attack_mod = 0
        self.player_pretend_def_mod = 0  # Specifically for Quick-Shot attack (Doesn't actually give defence bonus)

        self.mob_speed = 0
        self.mob_speed_mod = 0
        self.mob_crit_chance = 0
        self.mob_def_mod = 0
        self.mob_attack_mod = 0

        self.pc_goes_first = False
        self.pc_selected_move = None
        self.mob_selected_move = None

        # Status effects
        self.temp_effect_string = ""
        self.temp_effect_damage = 0
        self.temp_effect_duration = 0

    def combat_banner(self):
        """
        Generates a banner with stats for PC and Mob

        Cam's stats:
           | Atk: 10 | Def: 15 |
        HP |--------------------------------------------------| 50/50
        AP |--------------------------                        |  5/10

        Rat's Stats
           | Atk: 5 | Def: 7 |
        HP |---------------------------------------           | 20/25

        Combat Log:

        Cam attacks Rat for 1 damage

        :return:
        """

        # PC
        # Calculates how many dashes and spaces out of 50 to write for the healthbar
        pc_hp_quant = (self.player.current_health / self.player.max_health) * 50
        pc_hp_quant_negative = 50 - pc_hp_quant

        # To fix the weird issue where the wrong number of spaces appear
        if pc_hp_quant_negative > 0:
            pc_hp_quant_negative += 1

        # Same thing, but for AP instead
        pc_ap_quant = (self.player.ability_power / self.player.max_ability_power) * 50
        pc_ap_quant_negative = 50 - pc_ap_quant

        if pc_ap_quant_negative > 0:
            pc_ap_quant_negative += 1

        # Drawing the name, attack and defence stats for player
        pc_name = self.player.player_name + "'s stats:\n"
        pc_stats = "   | Atk: " + str(self.player.attack) + " | Def: " + str(self.player.defence) + " |\n"

        # Drawing the bars, simply using the string multiplier forcing the int type so we don't get a wrong type error
        pc_apbar = "AP " + "|" + "-" * int(pc_ap_quant) + " " * int(pc_ap_quant_negative) + "|\t"
        pc_healthbar = "HP " + "|" + "-" * int(pc_hp_quant) + " " * int(pc_hp_quant_negative) + "|\t"

        # Mob, with same stuff as above
        mob_hp_quant = (self.mob.current_health / self.mob.max_health) * 50
        mob_hp_quant_negative = 50 - mob_hp_quant

        if mob_hp_quant_negative > 0:
            mob_hp_quant_negative += 1

        mob_name = self.mob.name + "'s stats:\n"
        mob_stats = "   | Atk: " + str(self.mob.attack) + " | Def: " + str(self.mob.defence) + " |\n"

        mob_healthbar = "HP " + "|" + "-" * int(mob_hp_quant) + " " * int(mob_hp_quant_negative) + "|\t"

        pc_healthbar = pc_healthbar + str(self.player.current_health) + "/" + str(self.player.max_health) + "\n"
        pc_apbar = pc_apbar + str(self.player.ability_power) + "/" + str(self.player.max_ability_power) + "\n"
        mob_healthbar = mob_healthbar + str(self.mob.current_health) + "/" + str(self.mob.max_health) + "\n"

        # Making a long string to print out, to make it easy to swap them around / switch position
        banner_list = [pc_name, pc_stats, pc_healthbar, pc_apbar, "\n", mob_name, mob_stats, mob_healthbar]
        banner = "".join(banner_list)

        print(banner)

    # Menu options
    def combat_menu(self):
        options = ["Attack", "Defend", "Spells", "Items"]
        execu = self.attack, self.defend, self.spells, self.use_item
        util.menu_generator(options, execu)

    def attack(self):
        # todo cycle through equipped item, populating list of move methods
        """
        Gets list of attacks on equipped weapon
        Generates menu
        Adds cancel
        Sets the self.pc_selected_move key to the selected attack
        :return:
        """
        ev = util.EventHandler()

        # Get list of actions on equipped weapon
        attacks_list = []

        if self.player.weapon:
            for a in self.player.weapon.attacks:
                attacks_list.append(a)

        print("\nPlease select an option:")

        for i, a in enumerate(attacks_list):
            print(i + 1, ":", a.name)

        # Plays animation
        ev.wait_for_user()

        # Captures the keypress from the user
        player_input = ev.key_pressed
        print("\n\n", player_input)
        try:
            player_input = int(player_input) - 1
        except ValueError:
            print("Please select a valid option")
        util.clear()

        # Sets the pc selected move based on the player choice
        self.pc_selected_move = attacks_list[player_input]

    def defend(self):
        self.pc_selected_move = "defend"
        pass

    def spells(self):
        # todo cycle through spell list, populating a list of usable spell methods
        """
        Brings up spell menu
        sets selected move to Spell Method
        :return:
        """
        self.pc_selected_move = "spell"
        pass

    def use_item(self):

        # todo cycle through inventory of usable items, populating list of use methods
        """
        Brings up item menu
        Sets selected move to item Method
        :return:
        """
        self.pc_selected_move = "item"
        pass

    # Monster
    def mob_choice(self):
        """
        Enumerate the list of moves the monster has
        Add defend
        choose random choice (Some more likely than others)
        Possibly some sort of linear logic todo
        :return:
        """
        choices = []
        for i in self.mob.abilities:
            choices.append(i)
        choices.append("defend")

        self.mob_selected_move = random.choice(choices)

    # Combat

    def priority_decision(self):


        # Calculate speed
        # For 2 attackers
        if self.pc_selected_move != "defend" and self.mob_selected_move != "defend":
            pc_speed = (self.player.weapon.speed + self.pc_selected_move.speed +
                        (self.player.dexterity / self.player.strength) + random.choice(range(1 - 10)))
            mob_speed = ((self.mob.base_speed + self.mob_selected_move.speed) * self.mob.level) + \
                        random.choice(range(1 - 10))

            # Add defence bonuses
            self.player_def_mod += 1
            self.mob_def_mod += 1

        # For PC attacker and Mob defender
        if self.pc_selected_move != "defend" and self.mob_selected_move == "defend":
            pc_speed = (self.player_def_mod + self.player.defence + self.player.shield.defence_bonus +
                        self.player_pretend_def_mod + self.player.armor.defence_bonus + random.choice(range(1 - 10)))
            mob_speed = self.mob_def_mod + self.mob.defence + random.choice(range(1 - 10))

            # Add defence bonuses
            self.player_def_mod = 0
            self.mob_def_mod += 1

        # For Mob Attack and PC Defender
        if self.pc_selected_move == "defend" and self.mob_selected_move != "defend":
            pc_speed = self.player_def_mod + self.player.defence + self.player.shield.defence_bonus + \
                       self.player.armor.defence_bonus + random.choice(range(1 - 10))
            mob_speed = self.mob_def_mod + self.mob.defence + random.choice(range(1 - 10))

            # Add defence bonuses
            self.player_def_mod += 1
            self.mob_def_mod = 0

        # For 2 Defenders
        if self.pc_selected_move == "defend" and self.mob_selected_move == "defend":
            pc_speed = self.player_def_mod + self.player.defence + self.player.shield.defence_bonus + \
                       self.player.armor.defence_bonus + random.choice(range(1 - 10))
            mob_speed = self.mob_def_mod + self.mob.defence + random.choice(range(1 - 10))

            # Add defence bonuses
            self.player_def_mod = 0
            self.mob_def_mod = 0

        # Decide who goes first
        if pc_speed > mob_speed:
            self.pc_goes_first = True
        else:
            self.pc_goes_first = False

    def pc_turn(self):
        """
        todo
        Get PC selected move / action
        Execute if:
            First (Calculate bonus)
                Attack
                Defend
                Spell
                Item
            Second
                Attack
                Defend
                Spell
                Item
        Print results
        :return:
        """
        # If first
        if self.pc_goes_first:
            # Since the attack options are numerous, we'll decide the logic by whether it's "defend" or not
            pass

        # If Second
        else:
            pass

        # Print dialog, output to combat log

    def mob_turn(self):
        """
        todo
        Gets mob choice
        If it's first, calculate bonuses, etc.
        Execute attack / defence
        Print results / update banner
        :return:
        """
        # If first
        if not self.pc_goes_first:
            pass


        # If Second
        else:
            pass


    def encounter(self):
        """
        todo
        Announce attackers
        In a loop:
            Banner
            Mob choice
            Player choice
            Decide priority
            Execute 1st
            Execute 2nd
        Declare victor

        :return:
        """
        ev = util.EventHandler()
        # While at both HPs are not 0
        while (self.player.current_health > 0) and (self.mob.current_health > 0):
            # Display combat intro ( A fearsome rat appears out of the shadows \n)
            self.combat_banner()
            ev.wait_for_user()
            self.mob_selected_move = self.mob_choice()
            self.combat_menu()
            self.priority_decision()

            # Execute the appropriate method for who's going first
            if self.pc_goes_first:
                self.pc_turn()
            else:
                self.mob_turn()

        if self.player.current_health <= 0:
            print("The " + self.mob.name + " has killed you.\nYou died a terrible and painful death")
            return False
        if self.mob.current_health <= 0:
            print("The " + self.mob.name + " slumps over, dead.")
            print("You've gained " + str(self.mob.XP_given) + "XP")
            self.player.leveler(self.mob.XP_given)
            return True


